@@ -27,8 +27,11 @@
 // Description:
 //    POA functions
 
-// $Id: pyPOAFunc.cc,v 1.1.4.5 2006/01/19 17:28:44 dgrisby Exp $
+// $Id: pyPOAFunc.cc,v 1.1.4.6 2008/04/03 09:05:26 dgrisby Exp $
 // $Log: pyPOAFunc.cc,v $
+// Revision 1.1.4.6  2008/04/03 09:05:26  dgrisby
+// Leaks of some exception classes. Thanks Luke Deller.
+//
 // Revision 1.1.4.5  2006/01/19 17:28:44  dgrisby
 // Merge from omnipy2_develop.
 //
@@ -137,6 +140,7 @@ PyObject* raisePOAException(PyObject* pyPOA, const char* ename)
   OMNIORB_ASSERT(excc);
   PyObject* exci = PyEval_CallObject(excc, omniPy::pyEmptyTuple);
   PyErr_SetObject(excc, exci);
+  Py_DECREF(exci);
   return 0;
 }
 
@@ -294,6 +298,7 @@ extern "C" {
       OMNIORB_ASSERT(excc);
       PyObject* exci = PyObject_CallFunction(excc, (char*)"i", ex.index);
       PyErr_SetObject(excc, exci);
+      Py_DECREF(exci);
       return 0;
     }
     OMNIPY_CATCH_AND_HANDLE_SYSTEM_EXCEPTIONS
