@@ -27,14 +27,40 @@
 // Description:
 //    Implementation of Python servant object
 
-// $Id: pyServant.cc,v 1.1.2.15 2003/05/28 10:13:01 dgrisby Exp $
+// $Id: pyServant.cc,v 1.1.4.10 2008/02/01 16:29:17 dgrisby Exp $
 
 // $Log: pyServant.cc,v $
-// Revision 1.1.2.15  2003/05/28 10:13:01  dgrisby
-// Preliminary interceptor support. General clean-up.
+// Revision 1.1.4.10  2008/02/01 16:29:17  dgrisby
+// Error with implementation of operations with names clashing with
+// Python keywords.
 //
-// Revision 1.1.2.14  2003/03/28 17:44:36  dgrisby
-// Decref in wrong place led to crash.
+// Revision 1.1.4.9  2006/07/05 10:47:14  dgrisby
+// Propagate exceptions out of _default_POA.
+//
+// Revision 1.1.4.8  2005/11/09 12:33:32  dgrisby
+// Support POA LocalObjects.
+//
+// Revision 1.1.4.7  2005/07/22 17:41:07  dgrisby
+// Update from omnipy2_develop.
+//
+// Revision 1.1.4.6  2005/06/24 17:36:00  dgrisby
+// Support for receiving valuetypes inside Anys; relax requirement for
+// old style classes in a lot of places.
+//
+// Revision 1.1.4.5  2005/04/14 13:50:59  dgrisby
+// New traceTime, traceInvocationReturns functions; removal of omniORB::logf.
+//
+// Revision 1.1.4.4  2005/01/25 11:45:48  dgrisby
+// Merge from omnipy2_develop; set RPM version.
+//
+// Revision 1.1.4.3  2005/01/07 00:22:33  dgrisby
+// Big merge from omnipy2_develop.
+//
+// Revision 1.1.4.2  2003/05/20 17:10:24  dgrisby
+// Preliminary valuetype support.
+//
+// Revision 1.1.4.1  2003/03/23 21:51:57  dgrisby
+// New omnipy3_develop branch.
 //
 // Revision 1.1.2.13  2003/01/27 11:56:58  dgrisby
 // Correctly handle invalid returns from application code.
@@ -87,25 +113,35 @@
 
 // Implementation classes for ServantManagers and AdapterActivator
 
-class Py_ServantActivator: public virtual POA_PortableServer::ServantActivator,
-			   public virtual omniPy::Py_omniServant
+class Py_ServantActivatorSvt :
+  public virtual POA_PortableServer::ServantActivator,
+  public virtual omniPy::Py_omniServant
 {
 public:
-  Py_ServantActivator(PyObject* pysa, PyObject* opdict, const char* repoId);
-  virtual ~Py_ServantActivator();
+  Py_ServantActivatorSvt(PyObject* pysa, PyObject* opdict, const char* repoId)
+    : PY_OMNISERVANT_BASE(pysa, opdict, repoId), impl_(pysa) { }
+
+  virtual ~Py_ServantActivatorSvt() { }
 
   PortableServer::Servant incarnate(const PortableServer::ObjectId& oid,
-				    PortableServer::POA_ptr         poa);
+				    PortableServer::POA_ptr         poa)
+  {
+    return impl_.incarnate(oid, poa);
+  }
 
   void etherealize(const PortableServer::ObjectId& oid,
 		   PortableServer::POA_ptr         poa,
 		   PortableServer::Servant         serv,
 		   CORBA::Boolean                  cleanup_in_progress,
-		   CORBA::Boolean                  remaining_activations);
+		   CORBA::Boolean                  remaining_activations)
+  {
+    impl_.etherealize(oid, poa, serv, cleanup_in_progress,
+		      remaining_activations);
+  }
 
-  void*                   _ptrToInterface(const char* repoId);
+  void* _ptrToInterface(const char* repoId);
 
-  CORBA::Boolean          _is_a(const char* logical_type_id) {
+  CORBA::Boolean _is_a(const char* logical_type_id) {
     return PY_OMNISERVANT_BASE::_is_a(logical_type_id);
   }
   PortableServer::POA_ptr _default_POA() {
@@ -119,34 +155,43 @@ public:
   }
 
 private:
-  PyObject* pysa_;
+  omniPy::Py_ServantActivator impl_;
 
   // Not implemented
-  Py_ServantActivator(const Py_ServantActivator&);
-  Py_ServantActivator& operator=(const Py_ServantActivator&);
+  Py_ServantActivatorSvt(const Py_ServantActivatorSvt&);
+  Py_ServantActivatorSvt& operator=(const Py_ServantActivatorSvt&);
 };
 
-class Py_ServantLocator: public virtual POA_PortableServer::ServantLocator,
-			 public virtual omniPy::Py_omniServant
+class Py_ServantLocatorSvt :
+  public virtual POA_PortableServer::ServantLocator,
+  public virtual omniPy::Py_omniServant
 {
 public:
-  Py_ServantLocator(PyObject* pysl, PyObject* opdict, const char* repoId);
-  virtual ~Py_ServantLocator();
+  Py_ServantLocatorSvt(PyObject* pysl, PyObject* opdict, const char* repoId)
+    : PY_OMNISERVANT_BASE(pysl, opdict, repoId), impl_(pysl) { }
+
+  virtual ~Py_ServantLocatorSvt() { }
 
   PortableServer::Servant preinvoke(const PortableServer::ObjectId& oid,
 				    PortableServer::POA_ptr         poa,
 				    const char*                     operation,
-				    void*&                          cookie);
+				    void*&                          cookie)
+  {
+    return impl_.preinvoke(oid, poa, operation, cookie);
+  }
 
   void postinvoke(const PortableServer::ObjectId& oid,
 		  PortableServer::POA_ptr         poa,
 		  const char*                     operation,
 		  void*                           cookie,
-		  PortableServer::Servant         serv);
+		  PortableServer::Servant         serv)
+  {
+    impl_.postinvoke(oid, poa, operation, cookie, serv);
+  }
 
-  void*                   _ptrToInterface(const char* repoId);
+  void* _ptrToInterface(const char* repoId);
 
-  CORBA::Boolean          _is_a(const char* logical_type_id) {
+  CORBA::Boolean _is_a(const char* logical_type_id) {
     return PY_OMNISERVANT_BASE::_is_a(logical_type_id);
   }
   PortableServer::POA_ptr _default_POA() {
@@ -160,27 +205,33 @@ public:
   }
 
 private:
-  PyObject* pysl_;
+  omniPy::Py_ServantLocator impl_;
 
   // Not implemented
-  Py_ServantLocator(const Py_ServantLocator&);
-  Py_ServantLocator& operator=(const Py_ServantLocator&);
+  Py_ServantLocatorSvt(const Py_ServantLocatorSvt&);
+  Py_ServantLocatorSvt& operator=(const Py_ServantLocatorSvt&);
 };
 
 
-class Py_AdapterActivator: public virtual POA_PortableServer::AdapterActivator,
-			   public virtual omniPy::Py_omniServant
+class Py_AdapterActivatorSvt :
+  public virtual POA_PortableServer::AdapterActivator,
+  public virtual omniPy::Py_omniServant
 {
 public:
-  Py_AdapterActivator(PyObject* pyaa, PyObject* opdict, const char* repoId);
-  virtual ~Py_AdapterActivator();
+  Py_AdapterActivatorSvt(PyObject* pyaa, PyObject* opdict, const char* repoId)
+    : PY_OMNISERVANT_BASE(pyaa, opdict, repoId), impl_(pyaa) { }
+
+  virtual ~Py_AdapterActivatorSvt() { }
 
   CORBA::Boolean unknown_adapter(PortableServer::POA_ptr parent,
-				 const char*             name);
+				 const char*             name)
+  {
+    return impl_.unknown_adapter(parent, name);
+  }
 
-  void*                   _ptrToInterface(const char* repoId);
+  void* _ptrToInterface(const char* repoId);
 
-  CORBA::Boolean          _is_a(const char* logical_type_id) {
+  CORBA::Boolean _is_a(const char* logical_type_id) {
     return PY_OMNISERVANT_BASE::_is_a(logical_type_id);
   }
   PortableServer::POA_ptr _default_POA() {
@@ -194,11 +245,11 @@ public:
   }
 
 private:
-  PyObject* pyaa_;
+  omniPy::Py_AdapterActivator impl_;
 
   // Not implemented
-  Py_AdapterActivator(const Py_AdapterActivator&);
-  Py_AdapterActivator& operator=(const Py_AdapterActivator&);
+  Py_AdapterActivatorSvt(const Py_AdapterActivatorSvt&);
+  Py_AdapterActivatorSvt& operator=(const Py_AdapterActivatorSvt&);
 };
 
 
@@ -212,13 +263,12 @@ Py_omniServant::Py_omniServant(PyObject* pyservant, PyObject* opdict,
 {
   repoId_ = CORBA::string_dup(repoId);
 
-  OMNIORB_ASSERT(PyInstance_Check(pyservant));
   OMNIORB_ASSERT(PyDict_Check(opdict));
   Py_INCREF(pyservant_);
   Py_INCREF(opdict_);
 
   pyskeleton_ = PyObject_GetAttrString(pyservant_, (char*)"_omni_skeleton");
-  OMNIORB_ASSERT(pyskeleton_ && PyClass_Check(pyskeleton_));
+  OMNIORB_ASSERT(pyskeleton_);
 
   omniPy::setTwin(pyservant, (omniPy::Py_omniServant*)this, SERVANT_TWIN);
 }
@@ -314,30 +364,18 @@ Py_omniServant::_default_POA()
 	return PortableServer::POA::_duplicate(poa);
       }
       else {
-	if (omniORB::trace(1)) {
-	  omniORB::logger l;
-	  l <<
-	    "Python servant returned an invalid object from `_default_POA'.\n"
-	    "Returning Root POA\n";
-	}
+        omniORB::logs(1, "Python servant returned an invalid object from "
+                      "_default_POA.");
+        OMNIORB_THROW(BAD_PARAM, BAD_PARAM_WrongPythonType,
+                      CORBA::COMPLETED_NO);
       }      
     }
     else {
-      if (omniORB::trace(1)) {
-	{
-	  omniORB::logger l;
-	  l <<
-	    "Exception while trying to call _default_POA(). "
-	    "Returning Root POA\n";
-	}
-	PyErr_Print();
-      }
-      else
-	PyErr_Clear();
+      // The call raised a Python exception
+      omniORB::logs(1, "Python servant raised an exception in _default_POA.");
+      omniPy::handlePythonException();
     }
   }
-  CORBA::Object_var obj = omniPy::orb->resolve_initial_references("RootPOA");
-  return PortableServer::POA::_narrow(obj);
 }
 
 CORBA::Boolean
@@ -480,10 +518,14 @@ Py_omniServant::_dispatch(omniCallHandle& handle)
   }
   catch (omniPy::PyUserException& ex) {
     if (handle.iop_s()) {
-      {
+      try {
 	omniPy::InterpreterUnlocker _u;
 	handle.iop_s()->SendException(&ex);
       }
+      catch (...) {
+	ex.decrefPyException();
+	throw;
+      }
       ex.decrefPyException();
     }
     else
@@ -499,25 +541,31 @@ Py_omniServant::remote_dispatch(Py_omniCallDescriptor* pycd)
   const char* op     = pycd->op();
   PyObject*   method = PyObject_GetAttrString(pyservant_, (char*)op);
 
-  if (!method && omni::strMatch(op, "_interface")) {
-    PyErr_Clear();
-    method = PyObject_GetAttrString(pyservant_, (char*)"_get_interface");
-  }
   if (!method) {
-    if (omniORB::trace(1)) {
-      omniORB::logger l;
-      l << "Python servant for `" << repoId_ << "' has no method named `"
-	<< op << "'.\n";
-    }
     PyErr_Clear();
-    OMNIORB_THROW(NO_IMPLEMENT, NO_IMPLEMENT_NoPythonMethod,
-		  CORBA::COMPLETED_NO);
+    PyObject* word = PyDict_GetItemString(omniPy::pyomniORBwordMap, op);
+    if (word) {
+      // Keyword -- look up mangled name
+      method = PyObject_GetAttr(pyservant_, word);
+    }
+    else if (omni::strMatch(op, "_interface")) {
+      method = PyObject_GetAttrString(pyservant_, (char*)"_get_interface");
+    }
+    if (!method) {
+      if (omniORB::trace(1)) {
+	omniORB::logger l;
+	l << "Python servant for `" << repoId_ << "' has no method named `"
+	  << op << "'.\n";
+      }
+      PyErr_Clear();
+      OMNIORB_THROW(NO_IMPLEMENT, NO_IMPLEMENT_NoPythonMethod,
+		    CORBA::COMPLETED_NO);
+    }
   }
 
   PyObject* args   = pycd->args();
   PyObject* result = PyEval_CallObject(method, args);
   Py_DECREF(method);
-  Py_DECREF(args);
 
   if (result) {
     // No exception was thrown. Set the return value
@@ -528,9 +576,10 @@ Py_omniServant::remote_dispatch(Py_omniCallDescriptor* pycd)
     PyObject *etype, *evalue, *etraceback;
     PyObject *erepoId = 0;
     PyErr_Fetch(&etype, &evalue, &etraceback);
+    PyErr_NormalizeException(&etype, &evalue, &etraceback);
     OMNIORB_ASSERT(etype);
 
-    if (evalue && PyInstance_Check(evalue))
+    if (evalue)
       erepoId = PyObject_GetAttrString(evalue, (char*)"_NP_RepositoryId");
 
     if (!(erepoId && PyString_Check(erepoId))) {
@@ -543,10 +592,11 @@ Py_omniServant::remote_dispatch(Py_omniCallDescriptor* pycd)
 	PyErr_Restore(etype, evalue, etraceback);
 	PyErr_Print();
       }
+      else {
+        Py_DECREF(etype); Py_XDECREF(evalue); Py_XDECREF(etraceback);
+      }
       OMNIORB_THROW(UNKNOWN, UNKNOWN_PythonException, CORBA::COMPLETED_MAYBE);
     }
-    Py_DECREF(etype);
-    Py_XDECREF(etraceback);
 
     PyObject* exc_d = pycd->exc_d_;
 
@@ -557,7 +607,7 @@ Py_omniServant::remote_dispatch(Py_omniCallDescriptor* pycd)
       PyObject* edesc = PyDict_GetItem(exc_d, erepoId);
 
       if (edesc) {
-	Py_DECREF(erepoId);
+	Py_DECREF(erepoId); Py_DECREF(etype); Py_XDECREF(etraceback);
 	PyUserException ex(edesc, evalue, CORBA::COMPLETED_MAYBE);
 	ex._raise();
       }
@@ -566,12 +616,12 @@ Py_omniServant::remote_dispatch(Py_omniCallDescriptor* pycd)
     // Is it a LOCATION_FORWARD?
     if (omni::strMatch(PyString_AS_STRING(erepoId),
 		       "omniORB.LOCATION_FORWARD")) {
-      Py_DECREF(erepoId);
+      Py_DECREF(erepoId); Py_DECREF(etype); Py_XDECREF(etraceback);
       omniPy::handleLocationForward(evalue);
     }
 
     // System exception or unknown user exception
-    omniPy::produceSystemException(evalue, erepoId);
+    omniPy::produceSystemException(evalue, erepoId, etype, etraceback);
   }
 }
 
@@ -583,19 +633,26 @@ Py_omniServant::local_dispatch(Py_omniCallDescriptor* pycd)
   const char* op     = pycd->op();
   PyObject*   method = PyObject_GetAttrString(pyservant_, (char*)op);
 
-  if (!method && omni::strMatch(op, "_interface")) {
-    PyErr_Clear();
-    method = PyObject_GetAttrString(pyservant_, (char*)"_get_interface");
-  }
   if (!method) {
-    if (omniORB::trace(1)) {
-      omniORB::logger l;
-      l << "Python servant for `" << repoId_ << "' has no method named `"
-	<< op << "'.\n";
-    }
     PyErr_Clear();
-    OMNIORB_THROW(NO_IMPLEMENT, NO_IMPLEMENT_NoPythonMethod,
-		  CORBA::COMPLETED_NO);
+    PyObject* word = PyDict_GetItemString(omniPy::pyomniORBwordMap, op);
+    if (word) {
+      // Keyword -- look up mangled name
+      method = PyObject_GetAttr(pyservant_, word);
+    }
+    else if (omni::strMatch(op, "_interface")) {
+      method = PyObject_GetAttrString(pyservant_, (char*)"_get_interface");
+    }
+    if (!method) {
+      if (omniORB::trace(1)) {
+	omniORB::logger l;
+	l << "Python servant for `" << repoId_ << "' has no method named `"
+	  << op << "'.\n";
+      }
+      PyErr_Clear();
+      OMNIORB_THROW(NO_IMPLEMENT, NO_IMPLEMENT_NoPythonMethod,
+		    CORBA::COMPLETED_NO);
+    }
   }
 
   PyObject* in_d   = pycd->in_d_;
@@ -605,7 +662,7 @@ Py_omniServant::local_dispatch(Py_omniCallDescriptor* pycd)
   PyObject* exc_d  = pycd->exc_d_;
   PyObject* ctxt_d = pycd->ctxt_d_;
 
-  PyObject* args  = pycd->args();
+  PyObject* args   = pycd->args();
 
   // Copy args which would otherwise have reference semantics
   PyObject* argtuple = PyTuple_New(in_l + (ctxt_d ? 1 : 0));
@@ -628,14 +685,11 @@ Py_omniServant::local_dispatch(Py_omniCallDescriptor* pycd)
     }
   }
   catch (...) {
-    Py_DECREF(args);
     Py_DECREF(argtuple);
     Py_DECREF(method);
     throw;
   }
 
-  Py_DECREF(args);
-
   //
   // Do the call
 
@@ -690,9 +744,10 @@ Py_omniServant::local_dispatch(Py_omniCallDescriptor* pycd)
     PyObject *etype, *evalue, *etraceback;
     PyObject *erepoId = 0;
     PyErr_Fetch(&etype, &evalue, &etraceback);
+    PyErr_NormalizeException(&etype, &evalue, &etraceback);
     OMNIORB_ASSERT(etype);
 
-    if (evalue && PyInstance_Check(evalue))
+    if (evalue)
       erepoId = PyObject_GetAttrString(evalue, (char*)"_NP_RepositoryId");
 
     if (!(erepoId && PyString_Check(erepoId))) {
@@ -705,10 +760,11 @@ Py_omniServant::local_dispatch(Py_omniCallDescriptor* pycd)
 	PyErr_Restore(etype, evalue, etraceback);
 	PyErr_Print();
       }
+      else {
+        Py_DECREF(etype); Py_XDECREF(evalue); Py_XDECREF(etraceback);
+      }
       OMNIORB_THROW(UNKNOWN, UNKNOWN_PythonException, CORBA::COMPLETED_MAYBE);
     }
-    Py_DECREF(etype);
-    Py_XDECREF(etraceback);
 
     // Is it a user exception?
     if (exc_d != Py_None) {
@@ -717,7 +773,7 @@ Py_omniServant::local_dispatch(Py_omniCallDescriptor* pycd)
       PyObject* edesc = PyDict_GetItem(exc_d, erepoId);
 
       if (edesc) {
-	Py_DECREF(erepoId);
+	Py_DECREF(erepoId); Py_DECREF(etype); Py_XDECREF(etraceback);
 	PyUserException ex(edesc, evalue, CORBA::COMPLETED_MAYBE);
 	ex._raise();
       }
@@ -726,183 +782,20 @@ Py_omniServant::local_dispatch(Py_omniCallDescriptor* pycd)
     // Is it a LOCATION_FORWARD?
     if (omni::strMatch(PyString_AS_STRING(erepoId),
 		       "omniORB.LOCATION_FORWARD")) {
-      Py_DECREF(erepoId);
+      Py_DECREF(erepoId); Py_DECREF(etype); Py_XDECREF(etraceback);
       omniPy::handleLocationForward(evalue);
     }
 
     // System exception or unknown user exception
-    omniPy::produceSystemException(evalue, erepoId);
+    omniPy::produceSystemException(evalue, erepoId, etype, etraceback);
   }
 }
 
 
-// Implementation of Py_ServantActivator
-
-Py_ServantActivator::Py_ServantActivator(PyObject*   pysa,
-					 PyObject*   opdict,
-					 const char* repoId)
-  : PY_OMNISERVANT_BASE(pysa, opdict, repoId), pysa_(pysa)
-{
-  Py_INCREF(pysa);
-}
-
-Py_ServantActivator::~Py_ServantActivator()
-{
-  Py_DECREF(pysa_);
-}
-
-PortableServer::Servant
-Py_ServantActivator::incarnate(const PortableServer::ObjectId& oid,
-			       PortableServer::POA_ptr         poa)
-{
-  PyObject *method, *argtuple, *pyservant;
-  omnipyThreadCache::lock _t;
-
-  method = PyObject_GetAttrString(pysa_, (char*)"incarnate");
-  if (!method) {
-    PyErr_Clear();
-    OMNIORB_THROW(NO_IMPLEMENT, NO_IMPLEMENT_NoPythonMethod,
-		  CORBA::COMPLETED_MAYBE);
-  }
-  PortableServer::POA::_duplicate(poa);
-  argtuple = Py_BuildValue((char*)"s#N",
-			   (const char*)oid.NP_data(), oid.length(),
-			   omniPy::createPyPOAObject(poa));
-
-  // Do the up-call
-  pyservant = PyEval_CallObject(method, argtuple);
-  Py_DECREF(method);
-  Py_DECREF(argtuple);
-
-  if (pyservant) {
-    omniPy::Py_omniServant* servant = omniPy::getServantForPyObject(pyservant);
-    Py_DECREF(pyservant);
-
-    if (servant)
-      return servant;
-    else
-      OMNIORB_THROW(BAD_PARAM, BAD_PARAM_WrongPythonType, CORBA::COMPLETED_NO);
-  }
-  else {
-    // An exception of some sort was thrown
-    PyObject *etype, *evalue, *etraceback;
-    PyObject *erepoId = 0;
-    PyErr_Fetch(&etype, &evalue, &etraceback);
-    OMNIORB_ASSERT(etype);
-
-    if (evalue && PyInstance_Check(evalue))
-      erepoId = PyObject_GetAttrString(evalue, (char*)"_NP_RepositoryId");
-
-    if (!(erepoId && PyString_Check(erepoId))) {
-      Py_XDECREF(erepoId);
-      if (omniORB::trace(1)) {
-	{
-	  omniORB::logger l;
-	  l << "Caught an unexpected Python exception during up-call.\n";
-	}
-	PyErr_Restore(etype, evalue, etraceback);
-	PyErr_Print();
-      }
-      OMNIORB_THROW(UNKNOWN, UNKNOWN_PythonException, CORBA::COMPLETED_MAYBE);
-    }
-    Py_DECREF(etype);
-    Py_XDECREF(etraceback);
-
-    if (omni::strMatch(PyString_AS_STRING(erepoId),
-		       PortableServer::ForwardRequest::_PD_repoId)) {
-      Py_DECREF(erepoId);
-      PyObject* pyfr = PyObject_GetAttrString(evalue,
-					      (char*)"forward_reference");
-      Py_DECREF(evalue);
-      if (pyfr) {
-	CORBA::Object_ptr fr = (CORBA::Object_ptr)omniPy::getTwin(pyfr,
-								  OBJREF_TWIN);
-	if (fr) {
-	  PortableServer::ForwardRequest ex(fr);
-	  Py_DECREF(pyfr);
-	  throw ex;
-	}
-      }
-      else {
-	PyErr_Clear();
-	OMNIORB_THROW(BAD_PARAM, BAD_PARAM_WrongPythonType,
-		      CORBA::COMPLETED_NO);
-      }
-    }
-
-    // Is it a LOCATION_FORWARD?
-    if (omni::strMatch(PyString_AS_STRING(erepoId),
-		       "omniORB.LOCATION_FORWARD")) {
-      Py_DECREF(erepoId);
-      omniPy::handleLocationForward(evalue);
-    }
-
-    // System exception or unknown user exception
-    omniPy::produceSystemException(evalue, erepoId);
-  }
-  OMNIORB_ASSERT(0); // Never reach here
-  return 0;
-}
-
-
-void
-Py_ServantActivator::etherealize(const PortableServer::ObjectId& oid,
-				 PortableServer::POA_ptr poa,
-				 PortableServer::Servant serv,
-				 CORBA::Boolean          cleanup_in_progress,
-				 CORBA::Boolean          remaining_activations)
-{
-  PyObject *method, *argtuple, *result;
-  omnipyThreadCache::lock _t;
-
-  omniPy::Py_omniServant* pyos;
-  pyos = (omniPy::Py_omniServant*)serv->
-                                _ptrToInterface(omniPy::string_Py_omniServant);
-  if (!pyos) {
-    omniPy::InterpreterUnlocker _u;
-    serv->_remove_ref();
-    OMNIORB_THROW(OBJ_ADAPTER, OBJ_ADAPTER_IncompatibleServant,
-		  CORBA::COMPLETED_NO);
-  }
-
-  method = PyObject_GetAttrString(pysa_, (char*)"etherealize");
-  if (!method) {
-    PyErr_Clear();
-    omniPy::InterpreterUnlocker _u;
-    serv->_remove_ref();
-    OMNIORB_THROW(NO_IMPLEMENT, NO_IMPLEMENT_NoPythonMethod,
-		  CORBA::COMPLETED_NO);
-  }
-  PortableServer::POA::_duplicate(poa);
-  argtuple = Py_BuildValue((char*)"s#NNii",
-			   (const char*)oid.NP_data(), oid.length(),
-			   omniPy::createPyPOAObject(poa),
-			   pyos->pyServant(),
-			   (int)cleanup_in_progress,
-			   (int)remaining_activations);
-  // Do the up-call
-  result = PyEval_CallObject(method, argtuple);
-  Py_DECREF(method);
-  Py_DECREF(argtuple);
-
-  pyos->_locked_remove_ref();
-
-  if (result)
-    Py_DECREF(result);
-  else {
-    if (omniORB::trace(5))
-      omniORB::logf("omniORBpy: Servant etherealization raised an exception!");
-    if (omniORB::trace(10)) {
-      omniORB::logf("omniORBpy: Traceback follows:");
-      PyErr_Print();
-    }
-    else
-      PyErr_Clear();
-  }
-}
+// Py_ServantActivatorSvt
 
 void*
-Py_ServantActivator::_ptrToInterface(const char* repoId)
+Py_ServantActivatorSvt::_ptrToInterface(const char* repoId)
 {
   if (omni::ptrStrMatch(repoId, PortableServer::ServantActivator::_PD_repoId))
     return (PortableServer::_impl_ServantActivator*)this;
@@ -916,183 +809,10 @@ Py_ServantActivator::_ptrToInterface(const char* repoId)
   return 0;
 }
 
-
-// Implementation of Py_ServantLocator
-
-Py_ServantLocator::Py_ServantLocator(PyObject*   pysl,
-				     PyObject*   opdict,
-				     const char* repoId)
-  : PY_OMNISERVANT_BASE(pysl, opdict, repoId), pysl_(pysl)
-{
-  Py_INCREF(pysl);
-}
-
-Py_ServantLocator::~Py_ServantLocator()
-{
-  Py_DECREF(pysl_);
-}
-
-PortableServer::Servant
-Py_ServantLocator::preinvoke(const PortableServer::ObjectId& oid,
-			     PortableServer::POA_ptr         poa,
-			     const char*                     operation,
-			     void*&                          cookie)
-{
-  PyObject *method, *argtuple, *rettuple, *pyservant, *pycookie;
-  omnipyThreadCache::lock _t;
-
-  method = PyObject_GetAttrString(pysl_, (char*)"preinvoke");
-  if (!method) {
-    PyErr_Clear();
-    OMNIORB_THROW(NO_IMPLEMENT, NO_IMPLEMENT_NoPythonMethod,
-		  CORBA::COMPLETED_NO);
-  }
-  PortableServer::POA::_duplicate(poa);
-  argtuple = Py_BuildValue((char*)"s#Ns",
-			   (const char*)oid.NP_data(), oid.length(),
-			   omniPy::createPyPOAObject(poa),
-			   operation);
-
-  // Do the up-call
-  rettuple = PyEval_CallObject(method, argtuple);
-  Py_DECREF(method);
-  Py_DECREF(argtuple);
-
-  if (rettuple) {
-    if (PyTuple_Size(rettuple) != 2) {
-      Py_DECREF(rettuple);
-      OMNIORB_THROW(BAD_PARAM, BAD_PARAM_WrongPythonType, CORBA::COMPLETED_NO);
-    }
-    pyservant = PyTuple_GET_ITEM(rettuple, 0);
-    pycookie  = PyTuple_GET_ITEM(rettuple, 1);
-
-    omniPy::Py_omniServant* servant = omniPy::getServantForPyObject(pyservant);
-
-    if (servant) {
-      Py_INCREF(pycookie);
-      cookie = pycookie;
-      Py_DECREF(rettuple);
-      return servant;
-    }
-    else {
-      Py_DECREF(rettuple);
-      OMNIORB_THROW(BAD_PARAM, BAD_PARAM_WrongPythonType, CORBA::COMPLETED_NO);
-    }
-  }
-  else {
-    // An exception of some sort was thrown
-    PyObject *etype, *evalue, *etraceback;
-    PyObject *erepoId = 0;
-    PyErr_Fetch(&etype, &evalue, &etraceback);
-    OMNIORB_ASSERT(etype);
-
-    if (evalue && PyInstance_Check(evalue))
-      erepoId = PyObject_GetAttrString(evalue, (char*)"_NP_RepositoryId");
-
-    if (!(erepoId && PyString_Check(erepoId))) {
-      Py_XDECREF(erepoId);
-      if (omniORB::trace(1)) {
-	{
-	  omniORB::logger l;
-	  l << "Caught an unexpected Python exception during up-call.\n";
-	}
-	PyErr_Restore(etype, evalue, etraceback);
-	PyErr_Print();
-      }
-      OMNIORB_THROW(UNKNOWN, UNKNOWN_PythonException, CORBA::COMPLETED_MAYBE);
-    }
-    Py_DECREF(etype);
-    Py_XDECREF(etraceback);
-
-    if (omni::strMatch(PyString_AS_STRING(erepoId),
-		       PortableServer::ForwardRequest::_PD_repoId)) {
-      Py_DECREF(erepoId);
-      PyObject* pyfr = PyObject_GetAttrString(evalue,
-					      (char*)"forward_reference");
-      Py_DECREF(evalue);
-      if (pyfr) {
-	CORBA::Object_ptr fr = (CORBA::Object_ptr)omniPy::getTwin(pyfr,
-								  OBJREF_TWIN);
-	if (fr) {
-	  PortableServer::ForwardRequest ex(fr);
-	  Py_DECREF(pyfr);
-	  throw ex;
-	}
-      }
-      else {
-	PyErr_Clear();
-	OMNIORB_THROW(BAD_PARAM, BAD_PARAM_WrongPythonType,
-		      CORBA::COMPLETED_NO);
-      }
-    }
-    // Is it a LOCATION_FORWARD?
-    if (omni::strMatch(PyString_AS_STRING(erepoId),
-		       "omniORB.LOCATION_FORWARD")) {
-      Py_DECREF(erepoId);
-      omniPy::handleLocationForward(evalue);
-    }
-
-    // System exception or unknown user exception
-    omniPy::produceSystemException(evalue, erepoId);
-  }
-  OMNIORB_ASSERT(0); // Never reach here
-  return 0;
-}
-
-
-void
-Py_ServantLocator::postinvoke(const PortableServer::ObjectId& oid,
-			      PortableServer::POA_ptr         poa,
-			      const char*                     operation,
-			      void*                           cookie,
-			      PortableServer::Servant         serv)
-{
-  PyObject *method, *argtuple, *result;
-  omnipyThreadCache::lock _t;
-
-  omniPy::Py_omniServant* pyos;
-  pyos = (omniPy::Py_omniServant*)serv->
-                                _ptrToInterface(omniPy::string_Py_omniServant);
-  if (!pyos) {
-    omniPy::InterpreterUnlocker _u;
-    serv->_remove_ref();
-    OMNIORB_THROW(OBJ_ADAPTER, OBJ_ADAPTER_IncompatibleServant,
-		  CORBA::COMPLETED_NO);
-  }
-
-  method = PyObject_GetAttrString(pysl_, (char*)"postinvoke");
-  if (!method) {
-    PyErr_Clear();
-    omniPy::InterpreterUnlocker _u;
-    serv->_remove_ref();
-    OMNIORB_THROW(NO_IMPLEMENT, NO_IMPLEMENT_NoPythonMethod,
-		  CORBA::COMPLETED_NO);
-  }
-  PortableServer::POA::_duplicate(poa);
-  argtuple = Py_BuildValue((char*)"s#NsNN",
-			   (const char*)oid.NP_data(), oid.length(),
-			   omniPy::createPyPOAObject(poa),
-			   operation,
-			   (PyObject*)cookie,
-			   pyos->pyServant());
-  // Do the up-call
-  result = PyEval_CallObject(method, argtuple);
-  Py_DECREF(method);
-  Py_DECREF(argtuple);
-
-  pyos->_locked_remove_ref();
-
-  if (result) {
-    Py_DECREF(result);
-    return;
-  }
-  else {
-    omniPy::handlePythonException();
-  }
-}
+// Py_ServantLocatorSvt
 
 void*
-Py_ServantLocator::_ptrToInterface(const char* repoId)
+Py_ServantLocatorSvt::_ptrToInterface(const char* repoId)
 {
   if (omni::ptrStrMatch(repoId, PortableServer::ServantLocator::_PD_repoId))
     return (PortableServer::_impl_ServantLocator*)this;
@@ -1107,68 +827,10 @@ Py_ServantLocator::_ptrToInterface(const char* repoId)
 }
 
 
-// Implementation of Py_AdapterActivator
-
-Py_AdapterActivator::Py_AdapterActivator(PyObject*   pyaa,
-					 PyObject*   opdict,
-					 const char* repoId)
-  : PY_OMNISERVANT_BASE(pyaa, opdict, repoId), pyaa_(pyaa)
-{
-  Py_INCREF(pyaa);
-}
-
-Py_AdapterActivator::~Py_AdapterActivator()
-{
-  Py_DECREF(pyaa_);
-}
-
-CORBA::Boolean
-Py_AdapterActivator::unknown_adapter(PortableServer::POA_ptr parent,
-				     const char*             name)
-{
-  PyObject *method, *argtuple, *pyresult;
-  omnipyThreadCache::lock _t;
-
-  method = PyObject_GetAttrString(pyaa_, (char*)"unknown_adapter");
-  if (!method) {
-    PyErr_Clear();
-    OMNIORB_THROW(NO_IMPLEMENT, NO_IMPLEMENT_NoPythonMethod,
-		  CORBA::COMPLETED_NO);
-  }
-  PortableServer::POA::_duplicate(parent);
-  argtuple = Py_BuildValue((char*)"Ns",
-			   omniPy::createPyPOAObject(parent), name);
-
-  // Do the up-call
-  pyresult = PyEval_CallObject(method, argtuple);
-  Py_DECREF(method);
-  Py_DECREF(argtuple);
-
-  if (pyresult) {
-    if (!PyInt_Check(pyresult)) {
-      Py_DECREF(pyresult);
-      OMNIORB_THROW(BAD_PARAM, BAD_PARAM_WrongPythonType, CORBA::COMPLETED_NO);
-    }
-    CORBA::Boolean result = PyInt_AS_LONG(pyresult);
-    Py_DECREF(pyresult);
-    return result;
-  }
-  else {
-    if (omniORB::trace(5))
-      omniORB::logf("omniORBpy: AdapterActivator::unknown_adapter "
-		    "raised an exception!");
-    if (omniORB::trace(10)) {
-      omniORB::logf("omniORBpy: Traceback follows:");
-      PyErr_Print();
-    }
-    else
-      PyErr_Clear();
-  }
-  return 0;
-}
+// Py_AdapterActivatorSvt
 
 void*
-Py_AdapterActivator::_ptrToInterface(const char* repoId)
+Py_AdapterActivatorSvt::_ptrToInterface(const char* repoId)
 {
   if (omni::ptrStrMatch(repoId, PortableServer::AdapterActivator::_PD_repoId))
     return (PortableServer::_impl_AdapterActivator*)this;
@@ -1180,9 +842,6 @@ Py_AdapterActivator::_ptrToInterface(const char* repoId)
   return 0;
 }
 
-
-
-
 // Functions to create Py_omniServant objects
 
 static
@@ -1190,13 +849,13 @@ omniPy::Py_omniServant*
 newSpecialServant(PyObject* pyservant, PyObject* opdict, char* repoId)
 {
   if (omni::ptrStrMatch(repoId, PortableServer::ServantActivator::_PD_repoId))
-    return new Py_ServantActivator(pyservant, opdict, repoId);
+    return new Py_ServantActivatorSvt(pyservant, opdict, repoId);
 
   if (omni::ptrStrMatch(repoId, PortableServer::ServantLocator::_PD_repoId))
-    return new Py_ServantLocator(pyservant, opdict, repoId);
+    return new Py_ServantLocatorSvt(pyservant, opdict, repoId);
 
   if (omni::ptrStrMatch(repoId, PortableServer::AdapterActivator::_PD_repoId))
-    return new Py_AdapterActivator(pyservant, opdict, repoId);
+    return new Py_AdapterActivatorSvt(pyservant, opdict, repoId);
 
   OMNIORB_ASSERT(0);
   return 0;
@@ -1206,9 +865,6 @@ newSpecialServant(PyObject* pyservant, PyObject* opdict, char* repoId)
 omniPy::Py_omniServant*
 omniPy::getServantForPyObject(PyObject* pyservant)
 {
-  if (!PyInstance_Check(pyservant))
-    return 0;
-
   Py_omniServant* pyos;
 
   // Is there a Py_omniServant already?
@@ -1219,9 +875,7 @@ omniPy::getServantForPyObject(PyObject* pyservant)
   }
 
   // Is it an instance of the right class?
-  PyClassObject* pysc = ((PyInstanceObject*)pyservant)->in_class;
-
-  if (!PyClass_IsSubclass((PyObject*)pysc, omniPy::pyServantClass))
+  if (!omniPy::isInstance(pyservant, omniPy::pyServantClass))
     return 0;
 
   PyObject* opdict = PyObject_GetAttrString(pyservant, (char*)"_omni_op_d");
