@@ -27,8 +27,11 @@
 // Description:
 //    omniORB API functions
 
-// $Id: pyomniFunc.cc,v 1.1.4.7 2006/01/17 17:38:21 dgrisby Exp $
+// $Id: pyomniFunc.cc,v 1.1.4.8 2008/03/24 20:06:30 dgrisby Exp $
 // $Log: pyomniFunc.cc,v $
+// Revision 1.1.4.8  2008/03/24 20:06:30  dgrisby
+// Memory leak in minor code stringification.
+//
 // Revision 1.1.4.7  2006/01/17 17:38:21  dgrisby
 // Expose omniORB.setClientConnectTimeout function.
 //
@@ -691,10 +694,12 @@ extern "C" {
       return 0;
 
     pyrepoId = PyObject_GetAttrString(pyexc, (char*)"_NP_RepositoryId");
+    omniPy::PyRefHolder repoid_holder(pyrepoId);
     if (!pyrepoId)
       PyErr_Clear();
 
-    pyminor  = PyObject_GetAttrString(pyexc, (char*)"minor");
+    pyminor = PyObject_GetAttrString(pyexc, (char*)"minor");
+    omniPy::PyRefHolder minor_holder(pyminor);
     if (!pyminor)
       PyErr_Clear();
 
