@@ -28,8 +28,11 @@
 #    omniORB module -- omniORB specific things
 
 
-# $Id: __init__.py,v 1.30.2.21 2008/02/01 16:29:16 dgrisby Exp $
+# $Id: __init__.py,v 1.30.2.22 2008/08/21 10:53:55 dgrisby Exp $
 # $Log: __init__.py,v $
+# Revision 1.30.2.22  2008/08/21 10:53:55  dgrisby
+# Hook Thread.__stop instead of __delete. Thanks Luke Deller.
+#
 # Revision 1.30.2.21  2008/02/01 16:29:16  dgrisby
 # Error with implementation of operations with names clashing with
 # Python keywords.
@@ -1024,16 +1027,16 @@ class WorkerThread(threading.Thread):
 class omniThreadHook:
     def __init__(self, target):
         self.target            = target
-        self.target_del        = target._Thread__delete
-        target._Thread__delete = self.omni_thread_del
+        self.target_stop       = target._Thread__stop
+        target._Thread__stop   = self.omni_thread_stop
 
-    def omni_thread_del(self):
+    def omni_thread_stop(self):
         try:
             delattr(self.target, "__omni_thread")
-            del self.target._Thread__delete
+            del self.target._Thread__stop
         except AttributeError:
             pass
-        self.target_del()
+        self.target_stop()
 
 
 # System exception mapping.
